// Copyright (c) 2019 Palantir Technologies. All rights reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package periodic

import (
	"time"
)

type Option interface {
	apply(source *healthCheckSource)
}

type optionFn func(source *healthCheckSource)

func (fn optionFn) apply(source *healthCheckSource) {
	fn(source)
}

// WithInitialPoll configures the health check source to poll immediately instead of waiting for the duration of
// the check's specified retry interval.
func WithInitialPoll() Option {
	return optionFn(func(source *healthCheckSource) {
		source.initialPoll = true
	})
}

// WithStartupGracePeriod configures the health check source to transform all ERROR results into REPAIRING results
// for the first startupGracePeriod time window.
// The default value of the startup grace period is the window grace period.
func WithStartupGracePeriod(startupGracePeriod time.Duration) Option {
	return optionFn(func(source *healthCheckSource) {
		source.startupGracePeriod = startupGracePeriod
	})
}
